@@ -28,6 +28,7 @@
 
 #include "peas-i18n.h"
 #include "peas-engine.h"
+#include "peas-engine-priv.h"
 #include "peas-plugin-info-priv.h"
 #include "peas-plugin-loader.h"
 #include "peas-object-module.h"
@@ -91,7 +92,6 @@ struct _PeasEnginePrivate {
   GList *search_paths;
 
   GList *plugin_list;
-  GHashTable *loaders;
 
   guint in_dispose : 1;
 };
@@ -101,6 +101,9 @@ static void peas_engine_load_plugin_real   (PeasEngine     *engine,
 static void peas_engine_unload_plugin_real (PeasEngine     *engine,
                                             PeasPluginInfo *info);
 
+static gboolean shutdown = FALSE;
+static GHashTable *loaders = NULL;
+
 static void
 load_plugin_info (PeasEngine  *engine,
                   const gchar *filename,
@@ -276,48 +279,16 @@ loader_destroy (LoaderInfo *info)
   g_free (info);
 }
 
-static LoaderInfo *
-add_loader (PeasEngine       *engine,
-            const gchar      *loader_id,
-            PeasObjectModule *module,
-            PeasPluginLoader *loader)
-{
-  LoaderInfo *info;
-
-  info = g_new (LoaderInfo, 1);
-  info->loader = loader;
-  info->module = module;
-
-  g_hash_table_insert (engine->priv->loaders, g_strdup (loader_id), info);
-  return info;
-}
-
 static void
 peas_engine_init (PeasEngine *engine)
 {
-  /* Set the default engine pointer, for peas_engine_get_default().
-   * We only allow one single instance of a PeasEngine subclass. */
-  g_assert (default_engine == NULL);
-  default_engine = engine;
-  g_object_add_weak_pointer (G_OBJECT (engine), (gpointer *) &default_engine);
-
-  if (!g_module_supported ())
-    {
-      g_warning ("libpeas is not able to initialize the plugins engine.");
-      return;
-    }
-
   engine->priv = G_TYPE_INSTANCE_GET_PRIVATE (engine,
                                               PEAS_TYPE_ENGINE,
                                               PeasEnginePrivate);
 
   engine->priv->in_dispose = FALSE;
 
-  /* mapping from loadername -> loader object */
-  engine->priv->loaders = g_hash_table_new_full (hash_lowercase,
-                                                 (GEqualFunc) equal_lowercase,
-                                                 (GDestroyNotify) g_free,
-                                                 (GDestroyNotify) loader_destroy);
+  peas_engine_enable_loader (engine, "C");
 }
 
 static void
@@ -340,11 +311,46 @@ loader_garbage_collect (const gchar *id,
 void
 peas_engine_garbage_collect (PeasEngine *engine)
 {
-  g_hash_table_foreach (engine->priv->loaders,
+  g_return_if_fail (PEAS_IS_ENGINE (engine));
+
+  g_hash_table_foreach (loaders,
                         (GHFunc) loader_garbage_collect,
                         NULL);
 }
 
+static GObject *
+peas_engine_constructor (GType                  type,
+                         guint                  n_construct_params,
+                         GObjectConstructParam *construct_params)
+{
+  GObject *object;
+
+  /* We don't support calling PeasEngine API without module support */
+  if (!g_module_supported ())
+    {
+      g_error ("libpeas is not able to create the "
+               "plugins engine as modules are not supported.");
+    }
+
+  if (shutdown)
+    {
+      g_error ("libpeas cannot create a plugin engine "
+               "as it has been shutdown.");
+    }
+
+  object = G_OBJECT_CLASS (peas_engine_parent_class)->constructor (type,
+                                                                   n_construct_params,
+                                                                   construct_params);
+
+  if (default_engine == NULL)
+    {
+      default_engine = PEAS_ENGINE (object);
+      g_object_add_weak_pointer (object, (gpointer *) &default_engine);
+    }
+
+  return object;
+}
+
 static void
 peas_engine_set_property (GObject      *object,
                           guint         prop_id,
@@ -406,14 +412,16 @@ peas_engine_dispose (GObject *object)
         peas_engine_unload_plugin (engine, info);
     }
 
-  /* unref the loaders */
-  if (engine->priv->loaders != NULL)
-    {
-      g_hash_table_destroy (engine->priv->loaders);
-      engine->priv->loaders = NULL;
-    }
+  G_OBJECT_CLASS (peas_engine_parent_class)->dispose (object);
+}
 
-  /* and finally free the infos */
+static void
+peas_engine_finalize (GObject *object)
+{
+  PeasEngine *engine = PEAS_ENGINE (object);
+  GList *item;
+
+  /* free the infos */
   for (item = engine->priv->plugin_list; item; item = item->next)
     _peas_plugin_info_unref (PEAS_PLUGIN_INFO (item->data));
 
@@ -427,14 +435,6 @@ peas_engine_dispose (GObject *object)
       g_slice_free (SearchPath, sp);
     }
 
-  G_OBJECT_CLASS (peas_engine_parent_class)->dispose (object);
-}
-
-static void
-peas_engine_finalize (GObject *object)
-{
-  PeasEngine *engine = PEAS_ENGINE (object);
-
   g_list_free (engine->priv->plugin_list);
   g_list_free (engine->priv->search_paths);
 
@@ -447,6 +447,7 @@ peas_engine_class_init (PeasEngineClass *klass)
   GType the_type = G_TYPE_FROM_CLASS (klass);
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->constructor = peas_engine_constructor;
   object_class->set_property = peas_engine_set_property;
   object_class->get_property = peas_engine_get_property;
   object_class->dispose = peas_engine_dispose;
@@ -556,6 +557,12 @@ peas_engine_class_init (PeasEngineClass *klass)
   /* We are doing some global initialization here as there is currently no
    * global init function for libpeas. */
   peas_debug_init ();
+
+  /* mapping from loadername -> loader object */
+  loaders = g_hash_table_new_full (hash_lowercase,
+                                   (GEqualFunc) equal_lowercase,
+                                   (GDestroyNotify) g_free,
+                                   (GDestroyNotify) loader_destroy);
 }
 
 static PeasObjectModule *
@@ -597,101 +604,91 @@ try_to_open_loader_module (const gchar *loader_id,
   return module;
 }
 
-static LoaderInfo *
-load_plugin_loader (PeasEngine  *engine,
-                    const gchar *loader_id)
-{
-  guint i;
-  gchar *lc_loader_id;
-  PeasObjectModule *module;
-  PeasPluginLoader *loader;
-
-  /* We need to ensure we use the lowercase loader_id */
-  lc_loader_id = g_strdup (loader_id);
-  for (i = 0; lc_loader_id[i] != '\0'; ++i)
-    lc_loader_id[i] = g_ascii_tolower (lc_loader_id[i]);
-
-  module = try_to_open_loader_module (lc_loader_id, FALSE);
-  if (module == NULL)
-    module = try_to_open_loader_module (lc_loader_id, TRUE);
-
-  g_free (lc_loader_id);
-
-  if (module == NULL)
-    {
-      g_warning ("Loader '%s' could not be loaded", loader_id);
-      return add_loader (engine, loader_id, NULL, NULL);
-    }
-
-  loader = (PeasPluginLoader *) peas_object_module_create_object (module, PEAS_TYPE_PLUGIN_LOADER, 0, NULL);
-
-  if (loader == NULL || !PEAS_IS_PLUGIN_LOADER (loader))
-    {
-      g_warning ("Loader '%s' is not a valid PeasPluginLoader instance", loader_id);
-      if (loader != NULL && G_IS_OBJECT (loader))
-        g_object_unref (loader);
-      module = NULL;
-      loader = NULL;
-    }
-
-  g_type_module_unuse (G_TYPE_MODULE (module));
-
-  return add_loader (engine, loader_id, module, loader);
-}
-
 static PeasPluginLoader *
 get_plugin_loader (PeasEngine     *engine,
                    PeasPluginInfo *info)
 {
-  const gchar *loader_id;
   LoaderInfo *loader_info;
+  gchar *loader_id;
 
-  loader_id = info->loader;
-
-  loader_info = (LoaderInfo *) g_hash_table_lookup (engine->priv->loaders,
-                                                    loader_id);
+  loader_info = (LoaderInfo *) g_hash_table_lookup (loaders, info->loader);
 
+  /* The loader has not been enabled. */
   if (loader_info == NULL)
-    loader_info = load_plugin_loader (engine, loader_id);
+    return NULL;
+
+  /* The loader has already been loaded. */
+  if (loader_info->loader != NULL)
+    return loader_info->loader;
+
+  /* We need to ensure we use the lowercase loader_id */
+  loader_id = g_ascii_strdown (info->loader, -1);
+
+  loader_info->module = try_to_open_loader_module (loader_id, FALSE);
+  if (loader_info->module == NULL)
+    loader_info->module = try_to_open_loader_module (loader_id, TRUE);
+
+  g_free (loader_id);
+
+  if (loader_info->module == NULL)
+    {
+      g_hash_table_insert (loaders, loader_id, NULL);
+      return NULL;
+    }
+
+  loader_info->loader = PEAS_PLUGIN_LOADER (
+        peas_object_module_create_object (loader_info->module,
+                                          PEAS_TYPE_PLUGIN_LOADER,
+                                          0, NULL));
+
+  g_type_module_unuse (G_TYPE_MODULE (loader_info->module));
+
+  if (loader_info->loader == NULL ||
+      !PEAS_IS_PLUGIN_LOADER (loader_info->loader))
+    {
+      g_warning ("Loader '%s' is not a valid PeasPluginLoader instance",
+                 info->loader);
+
+      if (G_IS_OBJECT (loader_info->loader))
+        g_object_unref (loader_info->loader);
+
+      g_object_unref (loader_info->module);
+      g_hash_table_insert (loaders, info->loader, NULL);
+      return NULL;
+    }
 
   return loader_info->loader;
 }
 
 /**
- * peas_engine_disable_loader:
+ * peas_engine_enable_loader:
  * @engine: A #PeasEngine.
- * @loader_id: The id of the loader to inhibit.
+ * @loader_id: The id of the loader to enable.
  *
- * Disable a loader, avoiding using the plugins written using the
- * related language to be loaded. This method must be called before
- * any plugin relying on the loader @loader_id has been loaded.
+ * Enable a loader, enables a loader for plugins.
+ * The C plugin loader is always enabled.
  *
- * For instance, the following code will prevent any python plugin
+ * For instance, the following code will enable python plugin
  * from being loaded:
  * |[
- * peas_engine_disable_loader (engine, "python");
+ * peas_engine_enable_loader (engine, "python");
  * ]|
+ *
+ * Note: plugin loaders are shared across #PeasEngines so enabling
+ *       a loader on one #PeasEngine will enable it on all #PeasEngines.
  **/
 void
-peas_engine_disable_loader (PeasEngine  *engine,
-                            const gchar *loader_id)
+peas_engine_enable_loader (PeasEngine  *engine,
+                           const gchar *loader_id)
 {
-  LoaderInfo *loader_info;
-
   g_return_if_fail (PEAS_IS_ENGINE (engine));
+  g_return_if_fail (loader_id != NULL && *loader_id != '\0');
 
-  loader_info = (LoaderInfo *) g_hash_table_lookup (engine->priv->loaders,
-                                                    loader_id);
-  if (loader_info != NULL)
-    {
-      g_warning ("Loader '%s' cannot be disabled as it is already loaded",
-                 loader_id);
-      return;
-    }
+  if (g_hash_table_lookup_extended (loaders, loader_id, NULL, NULL))
+    return;
 
-  /* By adding a NULL loader, we simulate a failed load attempt, effectively
-   * disabling the loader for further use. */
-  add_loader (engine, loader_id, NULL, NULL);
+  /* The loader is loaded in get_plugin_loader() */
+  g_hash_table_insert (loaders, g_strdup (loader_id), g_new0 (LoaderInfo, 1));
 }
 
 /**
@@ -707,6 +704,8 @@ peas_engine_disable_loader (PeasEngine  *engine,
 const GList *
 peas_engine_get_plugin_list (PeasEngine *engine)
 {
+  g_return_val_if_fail (PEAS_IS_ENGINE (engine), NULL);
+
   return engine->priv->plugin_list;
 }
 
@@ -729,9 +728,14 @@ PeasPluginInfo *
 peas_engine_get_plugin_info (PeasEngine  *engine,
                              const gchar *plugin_name)
 {
-  GList *l = g_list_find_custom (engine->priv->plugin_list,
-                                 plugin_name,
-                                 (GCompareFunc) compare_plugin_info_and_name);
+  GList *l;
+
+  g_return_val_if_fail (PEAS_IS_ENGINE (engine), NULL);
+  g_return_val_if_fail (plugin_name != NULL, NULL);
+
+  l = g_list_find_custom (engine->priv->plugin_list,
+                          plugin_name,
+                          (GCompareFunc) compare_plugin_info_and_name);
 
   return l == NULL ? NULL : (PeasPluginInfo *) l->data;
 }
@@ -748,7 +752,7 @@ load_plugin (PeasEngine     *engine,
   if (peas_plugin_info_is_loaded (info))
     return TRUE;
 
-  if (!peas_plugin_info_is_available (info))
+  if (!peas_plugin_info_is_available (info, NULL))
     return FALSE;
 
   /* We set the plugin info as loaded before trying to load the dependencies,
@@ -762,11 +766,23 @@ load_plugin (PeasEngine     *engine,
       if (!dep_info)
         {
           g_warning ("Plugin not found: %s", dependencies[i]);
+          g_set_error (&info->error,
+                       PEAS_PLUGIN_INFO_ERROR,
+                       PEAS_PLUGIN_INFO_ERROR_DEP_NOT_FOUND,
+                       _("Dependency '%s' was not found"),
+                       dependencies[i]);
           goto error;
         }
 
       if (!peas_engine_load_plugin (engine, dep_info))
-        goto error;
+        {
+          g_set_error (&info->error,
+                       PEAS_PLUGIN_INFO_ERROR,
+                       PEAS_PLUGIN_INFO_ERROR_LOADING_FAILED,
+                       _("Dependency '%s' failed to load"),
+                       dep_info->name);
+          goto error;
+        }
     }
 
   loader = get_plugin_loader (engine, info);
@@ -775,12 +791,21 @@ load_plugin (PeasEngine     *engine,
     {
       g_warning ("Could not find loader '%s' for plugin '%s'",
                  info->loader, info->name);
+      g_set_error (&info->error,
+                   PEAS_PLUGIN_INFO_ERROR,
+                   PEAS_PLUGIN_INFO_ERROR_LOADER_NOT_FOUND,
+                   _("Plugin loader '%s' was not found"),
+                   info->loader);
       goto error;
     }
 
   if (!peas_plugin_loader_load (loader, info))
     {
       g_warning ("Error loading plugin '%s'", info->name);
+      g_set_error (&info->error,
+                   PEAS_PLUGIN_INFO_ERROR,
+                   PEAS_PLUGIN_INFO_ERROR_LOADING_FAILED,
+                   _("Failed to load"));
       goto error;
     }
 
@@ -817,9 +842,10 @@ gboolean
 peas_engine_load_plugin (PeasEngine     *engine,
                          PeasPluginInfo *info)
 {
+  g_return_val_if_fail (PEAS_IS_ENGINE (engine), FALSE);
   g_return_val_if_fail (info != NULL, FALSE);
 
-  if (!peas_plugin_info_is_available (info))
+  if (!peas_plugin_info_is_available (info, NULL))
     return FALSE;
 
   if (peas_plugin_info_is_loaded (info))
@@ -839,7 +865,7 @@ peas_engine_unload_plugin_real (PeasEngine     *engine,
   PeasPluginLoader *loader;
 
   if (!peas_plugin_info_is_loaded (info) ||
-      !peas_plugin_info_is_available (info))
+      !peas_plugin_info_is_available (info, NULL))
     return;
 
   /* We set the plugin info as unloaded before trying to unload the
@@ -884,6 +910,7 @@ gboolean
 peas_engine_unload_plugin (PeasEngine     *engine,
                            PeasPluginInfo *info)
 {
+  g_return_val_if_fail (PEAS_IS_ENGINE (engine), FALSE);
   g_return_val_if_fail (info != NULL, FALSE);
 
   if (!peas_plugin_info_is_loaded (info))
@@ -942,6 +969,7 @@ peas_engine_create_extensionv (PeasEngine     *engine,
 
   g_return_val_if_fail (PEAS_IS_ENGINE (engine), NULL);
   g_return_val_if_fail (info != NULL, NULL);
+  g_return_val_if_fail (peas_plugin_info_is_loaded (info), NULL);
 
   loader = get_plugin_loader (engine, info);
   return peas_plugin_loader_create_extension (loader, info, extension_type,
@@ -1051,15 +1079,19 @@ peas_engine_create_extension (PeasEngine     *engine,
  * Please note that the returned array is a newly allocated one: you will need
  * to free it using g_strfreev().
  *
- * Returns: (transfer full): A newly-allocated %NULL-terminated
- * array of strings, or %NULL.
+ * Returns: (transfer full) (array zero-terminated=1): A newly-allocated
+ * %NULL-terminated array of strings.
  */
 gchar **
 peas_engine_get_loaded_plugins (PeasEngine *engine)
 {
-  GArray *array = g_array_new (TRUE, FALSE, sizeof (gchar *));
+  GArray *array;
   GList *pl;
 
+  g_return_val_if_fail (PEAS_IS_ENGINE (engine), NULL);
+
+  array = g_array_new (TRUE, FALSE, sizeof (gchar *));
+
   for (pl = engine->priv->plugin_list; pl; pl = pl->next)
     {
       PeasPluginInfo *info = (PeasPluginInfo *) pl->data;
@@ -1072,16 +1104,7 @@ peas_engine_get_loaded_plugins (PeasEngine *engine)
         }
     }
 
-  /* Bug in GArray? */
-  if (array->len == 0)
-    {
-      g_array_free (array, TRUE);
-      return NULL;
-    }
-  else
-    {
-      return (gchar **) g_array_free (array, FALSE);
-    }
+  return (gchar **) g_array_free (array, FALSE);
 }
 
 static gboolean
@@ -1102,11 +1125,14 @@ string_in_strv (const gchar  *needle,
 /**
  * peas_engine_set_loaded_plugins:
  * @engine: A #PeasEngine.
- * @plugin_names: A %NULL-terminated array of plugin names.
+ * @plugin_names: (allow-none) (array zero-terminated=1): A %NULL-terminated
+ *  array of plugin names, or %NULL.
  *
  * Sets the list of loaded plugins for @engine. When this function is called,
  * the #PeasEngine will load all the plugins whose names are in @plugin_names,
  * and ensures all other active plugins are unloaded.
+ *
+ * If @plugin_names is %NULL, all plugins will be unloaded.
  */
 void
 peas_engine_set_loaded_plugins (PeasEngine   *engine,
@@ -1114,6 +1140,8 @@ peas_engine_set_loaded_plugins (PeasEngine   *engine,
 {
   GList *pl;
 
+  g_return_if_fail (PEAS_IS_ENGINE (engine));
+
   for (pl = engine->priv->plugin_list; pl; pl = pl->next)
     {
       PeasPluginInfo *info = (PeasPluginInfo *) pl->data;
@@ -1121,7 +1149,7 @@ peas_engine_set_loaded_plugins (PeasEngine   *engine,
       gboolean is_loaded;
       gboolean to_load;
 
-      if (!peas_plugin_info_is_available (info))
+      if (!peas_plugin_info_is_available (info, NULL))
         continue;
 
       module_name = peas_plugin_info_get_module_name (info);
@@ -1136,6 +1164,22 @@ peas_engine_set_loaded_plugins (PeasEngine   *engine,
     }
 }
 
+/**
+ * peas_engine_new:
+ *
+ * Return a new instance of #PeasEngine.
+ * If no default #PeasEngine has been instantiated yet,
+ * the first call of this function will set the default
+ * engine as the new instance of #PeasEngine.
+ *
+ * Returns: a new instance of #PeasEngine.
+ */
+PeasEngine *
+peas_engine_new (void)
+{
+  return PEAS_ENGINE (g_object_new (PEAS_TYPE_ENGINE, NULL));
+}
+
 /**
  * peas_engine_get_default:
  *
@@ -1148,8 +1192,30 @@ peas_engine_set_loaded_plugins (PeasEngine   *engine,
 PeasEngine *
 peas_engine_get_default (void)
 {
+  /* peas_engine_new() will cause the default to be set for us */
   if (default_engine == NULL)
-    return PEAS_ENGINE (g_object_new (PEAS_TYPE_ENGINE, NULL));
+    return peas_engine_new ();
 
   return default_engine;
 }
+
+/*
+ * peas_engine_shutdown:
+ *
+ * Frees memory shared by PeasEngines.
+ * No libpeas API should be called after calling this.
+ */
+void
+peas_engine_shutdown (void)
+{
+  if (shutdown)
+    return;
+
+  shutdown = TRUE;
+
+  if (loaders != NULL)
+    {
+      g_hash_table_destroy (loaders);
+      loaders = NULL;
+    }
+}
