@@ -59,7 +59,7 @@ create_main_window (void)
   GtkWidget *button;
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), NULL);
+  g_signal_connect (window, "delete-event", G_CALLBACK (gtk_main_quit), NULL);
   gtk_container_set_border_width (GTK_CONTAINER (window), 6);
   gtk_window_set_title (GTK_WINDOW (window), "Peas Demo");
 
@@ -70,7 +70,7 @@ create_main_window (void)
   box = gtk_vbox_new (FALSE, 6);
   gtk_container_add (GTK_CONTAINER (window), box);
 
-  manager = peas_gtk_plugin_manager_new ();
+  manager = peas_gtk_plugin_manager_new (peas_engine_get_default ());
   gtk_box_pack_start (GTK_BOX (box), manager, TRUE, TRUE, 0);
 
   button_box = gtk_hbutton_box_new ();
@@ -125,6 +125,9 @@ main (int    argc,
   peas_engine_add_search_path (engine, plugin_dir, plugin_dir);
   g_free (plugin_dir);
 
+  peas_engine_enable_loader (engine, "python");
+  peas_engine_enable_loader (engine, "seed");
+
   if (run_from_build_dir)
     peas_engine_add_search_path (engine, "./plugins", NULL);
   else
@@ -138,7 +141,7 @@ main (int    argc,
 
   gtk_main ();
 
-  g_object_unref (engine);
+  gtk_widget_destroy (main_window);
 
   return 0;
 }
